@@ -67,6 +67,8 @@ class MainWindow:
 		self.tree.get_object('move_up_button').set_sensitive(False)
 		self.tree.get_object('move_down_button').set_sensitive(False)
 		self.tree.get_object('new_separator_button').set_sensitive(False)
+		self.tree.get_object('properties_button').set_sensitive(False)
+		self.tree.get_object('delete_button').set_sensitive(False)
 		accelgroup = Gtk.AccelGroup()
 		keyval, modifier = Gtk.accelerator_parse('<Ctrl>Z')
 		accelgroup.connect(keyval, modifier, Gtk.AccelFlags.VISIBLE, self.on_mainwindow_undo)
@@ -273,14 +275,14 @@ class MainWindow:
 	#this is a little timeout callback to insert new items after
 	#mate-desktop-item-edit has finished running
 	def waitForNewItemProcess(self, process, parent_id, file_path):
-		if process.poll() != None:
+		if process.poll() is not None:
 			if os.path.isfile(file_path):
 				self.editor.insertExternalItem(os.path.split(file_path)[1], parent_id)
 			return False
 		return True
 
 	def waitForNewMenuProcess(self, process, parent_id, file_path):
-		if process.poll() != None:
+		if process.poll() is not None:
 			#hack for broken mate-desktop-item-edit
 			broken_path = os.path.join(os.path.split(file_path)[0], '.directory')
 			if os.path.isfile(broken_path):
@@ -292,7 +294,7 @@ class MainWindow:
 
 	#this callback keeps you from editing the same item twice
 	def waitForEditProcess(self, process, file_path):
-		if process.poll() != None:
+		if process.poll() is not None:
 			self.edit_pool.remove(file_path)
 			return False
 		return True
@@ -375,15 +377,7 @@ class MainWindow:
 			file_path = os.path.join(util.getUserItemPath(), item.get_desktop_file_id())
 			file_type = 'Item'
 		elif item.get_type() == matemenu.TYPE_DIRECTORY:
-			if item.get_desktop_file_path() == None:
-				file_path = util.getUniqueFileId('mozo-made', '.directory')
-				parser = util.DesktopParser(file_path, 'Directory')
-				parser.set('Name', item.get_name())
-				parser.set('Comment', item.get_comment())
-				parser.set('Icon', item.get_icon())
-				parser.write(open(file_path))
-			else:
-				file_path = os.path.join(util.getUserDirectoryPath(), os.path.split(item.get_desktop_file_path())[1])
+			file_path = os.path.join(util.getUserDirectoryPath(), os.path.split(item.get_desktop_file_path())[1])
 			file_type = 'Menu'
 
 		if not os.path.isfile(file_path):
@@ -425,16 +419,17 @@ class MainWindow:
 				context.finish(False, False, etime)
 				return False
 			if str(selection.get_target()) in ('MOZO_ITEM_ROW', 'MOZO_MENU_ROW'):
-				if self.drag_data == None:
+				if self.drag_data is None:
 					return False
 				item = self.drag_data
 				new_parent = menus[path][2]
-				treeview.get_selection().select_path(path)
 				if item.get_type() == matemenu.TYPE_ENTRY:
 					self.editor.copyItem(item, new_parent)
 				elif item.get_type() == matemenu.TYPE_DIRECTORY:
-					if self.editor.moveMenu(item, new_parent) == False:
+					if not self.editor.moveMenu(item, new_parent):
 						self.loadUpdates()
+				elif item.get_type() == matemenu.TYPE_SEPARATOR:
+					self.editor.moveSeparator(item, new_parent)
 				else:
 					context.finish(False, False, etime) 
 				context.finish(True, True, etime)
@@ -454,29 +449,24 @@ class MainWindow:
 		items, iter = treeview.get_selection().get_selected()
 		if iter is None:
 			return
+
 		item = items[iter][3]
 		self.tree.get_object('edit_delete').set_sensitive(True)
 		self.tree.get_object('new_separator_button').set_sensitive(True)
-		if self.editor.canRevert(item):
-			self.tree.get_object('edit_revert_to_original').set_sensitive(True)
-		else:
-			self.tree.get_object('edit_revert_to_original').set_sensitive(False)
-		if not item.get_type() == matemenu.TYPE_SEPARATOR:
-			self.tree.get_object('edit_properties').set_sensitive(True)
-		else:
-			self.tree.get_object('edit_properties').set_sensitive(False)
+		self.tree.get_object('delete_button').set_sensitive(True)
 
-		# If first item...
-		if items.get_path(iter).get_indices()[0] == 0:
-			self.tree.get_object('move_up_button').set_sensitive(False)
-		else:
-			self.tree.get_object('move_up_button').set_sensitive(True)
+		can_revert = self.editor.canRevert(item)
+		self.tree.get_object('edit_revert_to_original').set_sensitive(can_revert)
 
-		# If last item...
-		if items.get_path(iter).get_indices()[0] == (len(items)-1):
-			self.tree.get_object('move_down_button').set_sensitive(False)
-		else:
-			self.tree.get_object('move_down_button').set_sensitive(True)
+		can_edit = not item.get_type() == matemenu.TYPE_SEPARATOR
+		self.tree.get_object('edit_properties').set_sensitive(can_edit)
+		self.tree.get_object('properties_button').set_sensitive(can_edit)
+
+		index = items.get_path(iter).get_indices()[0]
+		can_go_up = index > 0
+		can_go_down = index < len(items) - 1
+		self.tree.get_object('move_up_button').set_sensitive(can_go_up)
+		self.tree.get_object('move_down_button').set_sensitive(can_go_down)
 
 	def on_item_tree_row_activated(self, treeview, path, column):
 		self.on_edit_properties_activate(None)
@@ -490,7 +480,7 @@ class MainWindow:
 			button = event.button
 			event_time = event.time
 			info = item_tree.get_path_at_pos(int(event.x), int(event.y))
-			if info != None:
+			if info is not None:
 				path, col, cellx, celly = info
 				item_tree.grab_focus()
 				item_tree.set_cursor(path, col, 0)
@@ -511,33 +501,45 @@ class MainWindow:
 
 	def on_item_tree_drag_data_received(self, treeview, context, x, y, selection, info, etime):
 		items = treeview.get_model()
-		types = (Gtk.TreeViewDropPosition.BEFORE,	Gtk.TreeViewDropPosition.INTO_OR_BEFORE)
+		types_before = (Gtk.TreeViewDropPosition.BEFORE, Gtk.TreeViewDropPosition.INTO_OR_BEFORE)
+		types_into = (Gtk.TreeViewDropPosition.INTO_OR_BEFORE, Gtk.TreeViewDropPosition.INTO_OR_AFTER)
+		types_after = (Gtk.TreeViewDropPosition.AFTER, Gtk.TreeViewDropPosition.INTO_OR_AFTER)
 		if str(selection.get_target()) == 'MOZO_ITEM_ROW':
 			drop_info = treeview.get_dest_row_at_pos(x, y)
 			before = None
 			after = None
-			if self.drag_data == None:
+			if self.drag_data is None:
 				return False
 			item = self.drag_data
+			# by default we assume, that the items stays in the same menu
+			destination = item.get_parent()
 			if drop_info:
 				path, position = drop_info
-				if position in types:
-					before = items[path][3]
+				target = items[path][3]
+				# move the item to the directory, if the item was dropped into it
+				if (target.get_type() == matemenu.TYPE_DIRECTORY) and (position in types_into):
+					# append the selected item to the choosen menu
+					destination = target
+				elif position in types_before:
+					before = target
+				elif position in types_after:
+					after = target
 				else:
-					after = items[path][3]
+					# this does not happen
+					pass
 			else:
 				path = (len(items) - 1,)
 				after = items[path][3]
 			if item.get_type() == matemenu.TYPE_ENTRY:
-				self.editor.moveItem(item, item.get_parent(), before, after)
+				self.editor.moveItem(item, destination, before, after)
 			elif item.get_type() == matemenu.TYPE_DIRECTORY:
-				if self.editor.moveMenu(item, item.get_parent(), before, after) == False:
+				if not self.editor.moveMenu(item, destination, before, after):
 					self.loadUpdates()
 			elif item.get_type() == matemenu.TYPE_SEPARATOR:
-				self.editor.moveSeparator(item, item.get_parent(), before, after)
+				self.editor.moveSeparator(item, destination, before, after)
 			context.finish(True, True, etime)
 		elif str(selection.get_target()) == 'text/plain':
-			if selection.data == None:
+			if selection.data is None:
 				return False
 			menus, iter = self.tree.get_object('menu_tree').get_selection().get_selected()
 			parent = menus[iter][2]
@@ -546,7 +548,7 @@ class MainWindow:
 			after = None
 			if drop_info:
 				path, position = drop_info
-				if position in types:
+				if position in types_before:
 					before = items[path][3]
 				else:
 					after = items[path][3]
@@ -561,13 +563,14 @@ class MainWindow:
 			content_type = file_info.get_content_type()
 			if content_type == 'application/x-desktop':
 				input_stream = myfile.read()
-				(fd, tmppath) = tempfile.mkstemp(prefix='mozo-dnd', suffix='.desktop')
-				with open(fd, "w") as f:
-					f.write(input_stream.read())
-				parser = util.DesktopParser(tmppath)
-				self.editor.createItem(parent, parser.get('Icon'), parser.get('Name', self.editor.locale), parser.get('Comment', self.editor.locale), parser.get('Exec'), parser.get('Terminal'), before, after)
+				keyfile = GLib.KeyFile()
+				keyfile.load_from_data(input_stream.read())
+				self.editor.createItem(parent, before, after, KeyFile=keyfile)
 			elif content_type in ('application/x-shellscript', 'application/x-executable'):
-				self.editor.createItem(parent, None, os.path.split(file_path)[1].strip(), None, file_path.replace('file://', '').strip(), False, before, after)
+				self.editor.createItem(parent, before, after,
+				                       Name=os.path.split(file_path)[1].strip(),
+				                       Exec=file_path.replace('file://', '').strip(),
+				                       Terminal=False)
 		self.drag_data = None
 
 	def on_item_tree_key_press_event(self, item_tree, event):
@@ -634,6 +637,12 @@ class MainWindow:
 			pass
 		GLib.timeout_add(10, self.quit)
 
+	def on_properties_button_clicked(self, button):
+		self.on_edit_properties_activate(None)
+
+	def on_delete_button_clicked(self, button):
+		self.on_edit_delete_activate(None)
+
 	def on_style_updated(self, *args):
 		self.loadUpdates()
 
